#include <platform_stdlib.h>
#include <platform_opts.h>
#include <hal_adc.h>
#include <gpio_api.h>   // mbed
#include <sys_api.h>
#include <nfc_api.h>
#include "log_service.h"

enum{
	AT_DBG_OFF = 0,
	AT_DBG_ALWAYS,
	AT_DBG_ERROR,
	AT_DBG_WARNING,
	AT_DBG_INFO
};

#define AT_BIT(n)           (1<<n)
#define AT_FLAG_DUMP        AT_BIT(0)
#define AT_FLAG_EDIT        AT_BIT(1)
#define AT_FLAG_ADC         AT_BIT(2)
#define AT_FLAG_GPIO        AT_BIT(3)
#define AT_FLAG_OTA         AT_BIT(4)
#define AT_FLAG_NFC         AT_BIT(5)

u8	gDbgLevel = AT_DBG_ERROR;
u32	gDbgFlag = 0xFFFFFFFF;

#define AT_PRINTK(fmt, args...)	printf("\r\n" fmt,## args)
#define AT_DBG_MSG(flag, level, fmt, args...)				\
		if(((flag) & gDbgFlag) && (level <= gDbgLevel))		\
		{													\
			AT_PRINTK(fmt,## args);							\
		}
		
#define MAX_ARGC	6

extern u32 ConfigDebugErr;
extern u32 ConfigDebugInfo;
extern u32 ConfigDebugWarn;
extern u32 CmdDumpWord(IN u16 argc, IN u8 *argv[]);
extern u32 CmdWriteWord(IN u16 argc, IN u8 *argv[]);

//----------------------------------------------------------------------------------------
int sys_parse_param(char *buf, char **argv)
{
	int argc = 1;
	while((argc < MAX_ARGC) && (*buf != '\0')) {
          while((*buf == '[')||(*buf == ' '))
            buf ++;         
          if((*buf == ']')||(*buf == '\0'))
            break;
          argv[argc] = buf;
          argc ++;
          buf ++;

          while((*buf != ',')&&(*buf != '[')&&(*buf != ']')&&(*buf != '\0'))
                  buf ++;

          while((*buf == ',')||(*buf == '[')||(*buf == ']')) {
                  *buf = '\0';
                  buf ++;
          }
	}
	return argc;
}

void fATSD(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_DBG_MSG(AT_FLAG_DUMP, AT_DBG_ALWAYS, "[ATSD]: _AT_SYSTEM_DUMP_REGISTER_");
	if(!arg){
		AT_DBG_MSG(AT_FLAG_DUMP, AT_DBG_ALWAYS, "[ATSD] Usage: ATSD=REGISTER");
		return;
	}
	argc = sys_parse_param(arg, argv);
	if(argc == 2 || argc == 3)
		CmdDumpWord(argc-1, (unsigned char**)(argv+1));
}

void fATSE(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_DBG_MSG(AT_FLAG_EDIT, AT_DBG_ALWAYS, "[ATSE]: _AT_SYSTEM_EDIT_REGISTER_");
	if(!arg){
		AT_DBG_MSG(AT_FLAG_EDIT, AT_DBG_ALWAYS, "[ATSE] Usage: ATSE=REGISTER[VALUE]");
		return;
	}
	argc = sys_parse_param(arg, argv);
	if(argc == 3)
		CmdWriteWord(argc-1, (unsigned char**)(argv+1));
}

void fATSA(void *arg)
{
	PSAL_ADC_HND pADCVeriHnd = NULL;
	u32 AdcLocalTmp[2];
	u32 tConfigDebugInfo = ConfigDebugInfo;
	int argc = 0, channel;
	char *argv[MAX_ARGC] = {0};
	
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA]: _AT_SYSTEM_ADC_TEST_");
	if(!arg){
		AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA] Usage: ATSA=CHANNEL(0~2)");
		return;
	}else{
		argc = sys_parse_param(arg, argv);
		channel = atoi(argv[1]);
		if(argc != 2 || channel < 0 || channel > 2){
			AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA] Usage: ATSA=CHANNEL(0~2)");
			return;
		}
	}
	
	// Remove debug info massage
	ConfigDebugInfo = 0;
	
	pADCVeriHnd = RtkADCGetSalHnd(0);
	pADCVeriHnd->DevNum     = 0;
    RtkADCLoadDefault(pADCVeriHnd);
	
	pADCVeriHnd->pInitDat->ADCBurstSz       =   8;
    pADCVeriHnd->pInitDat->ADCOneShotTD     =   2;
    //pADCVeriHnd->pInitDat->ADCAudioEn       =   ADC_FEATURE_ENABLED;
    //pADCVeriHnd->pInitDat->ADCOverWREn      =   1;
    pADCVeriHnd->OpType                     =   ADC_RDREG_TYPE;
    
    RtkADCInit(pADCVeriHnd);
	RtkADCReceiveBuf(pADCVeriHnd,&AdcLocalTmp[0]);
	RtkADCDeInit(pADCVeriHnd);
	RtkADCFreeSalHnd(pADCVeriHnd);
	// Recover debug info massage
	ConfigDebugInfo = tConfigDebugInfo;
	
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_INFO, "AdcTmp0:0x%08X, AdcTmp1:0x%08X", AdcLocalTmp[0], AdcLocalTmp[1]);
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_INFO, "Adc Ch0:0x%04X", (AdcLocalTmp[0]&0x0000FFFF)>>4);
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_INFO, "Adc Ch1:0x%04X", (AdcLocalTmp[0]&0xFFFF0000)>>20);
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_INFO, "Adc Ch2:0x%04X", (AdcLocalTmp[1]&0x0000FFFF)>>4);
	AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_INFO, "Adc Ch3:0x%04X", (AdcLocalTmp[1]&0xFFFF0000)>>20);
	if(channel == 0)
		AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA] A0 = 0x%04X", (AdcLocalTmp[0]&0xFFFF0000)>>20);
	if(channel == 1)
		AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA] A1 = 0x%04X", (AdcLocalTmp[1]&0x0000FFFF)>>4);
	if(channel == 2)
		AT_DBG_MSG(AT_FLAG_ADC, AT_DBG_ALWAYS, "[ATSA] A2 = 0x%04X", (AdcLocalTmp[1]&0xFFFF0000)>>20);
}

void fATSG(void *arg)
{
    gpio_t gpio_test;
    int argc = 0, val;
	char *argv[MAX_ARGC] = {0}, port, num;
	PinName pin = NC;
	u32 tConfigDebugInfo = ConfigDebugInfo;
    
	AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSG]: _AT_SYSTEM_GPIO_TEST_");
	if(!arg){
		AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSG] Usage: ATSG=PINNAME(ex:A0)");
		return;
	}else{
		argc = sys_parse_param(arg, argv);
		if(argc != 2){
			AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSG] Usage: ATSG=PINNAME(ex:A0)");
			return;
		}
	}
	port = argv[1][0];
	num = argv[1][1];
	if(port >= 'a' && port <= 'z')
		port -= ('a' - 'A');
	if(num >= 'a' && num <= 'z')
		num -= ('a' - 'A');
	switch(port){
		case 'A':
			switch(num){
				case '0': pin = PA_0; break; case '1': pin = PA_1; break; case '2': pin = PA_2; break; case '3': pin = PA_3; break;
				case '4': pin = PA_4; break; case '5': pin = PA_5; break; case '6': pin = PA_6; break; case '7': pin = PA_7; break;
			}
			break;
		case 'B':
			switch(num){
				case '0': pin = PB_0; break; case '1': pin = PB_1; break; case '2': pin = PB_2; break; case '3': pin = PB_3; break;
				case '4': pin = PB_4; break; case '5': pin = PB_5; break; case '6': pin = PB_6; break; case '7': pin = PB_7; break;
			}
			break;
		case 'C':
			switch(num){
				case '0': pin = PC_0; break; case '1': pin = PC_1; break; case '2': pin = PC_2; break; case '3': pin = PC_3; break;
				case '4': pin = PC_4; break; case '5': pin = PC_5; break; case '6': pin = PC_6; break; case '7': pin = PC_7; break;
				case '8': pin = PC_8; break; case '9': pin = PC_9; break;
			}
			break;
		case 'D':
			switch(num){
				case '0': pin = PD_0; break; case '1': pin = PD_1; break; case '2': pin = PD_2; break; case '3': pin = PD_3; break;
				case '4': pin = PD_4; break; case '5': pin = PD_5; break; case '6': pin = PD_6; break; case '7': pin = PD_7; break;
				case '8': pin = PD_8; break; case '9': pin = PD_9; break;
			}
			break;
		case 'E':
			switch(num){
				case '0': pin = PE_0; break; case '1': pin = PE_1; break; case '2': pin = PE_2; break; case '3': pin = PE_3; break;
				case '4': pin = PE_4; break; case '5': pin = PE_5; break; case '6': pin = PE_6; break; case '7': pin = PE_7; break;
				case '8': pin = PE_8; break; case '9': pin = PE_9; break; case 'A': pin = PE_A; break;
			}
			break;
		case 'F':
			switch(num){
				case '0': pin = PF_0; break; case '1': pin = PF_1; break; case '2': pin = PF_2; break; case '3': pin = PF_3; break;
				case '4': pin = PF_4; break; case '5': pin = PF_5; break;
			}
			break;
		case 'G':
			switch(num){
				case '0': pin = PG_0; break; case '1': pin = PG_1; break; case '2': pin = PG_2; break; case '3': pin = PG_3; break;
				case '4': pin = PG_4; break; case '5': pin = PG_5; break; case '6': pin = PG_6; break; case '7': pin = PG_7; break;
			}
			break;
		case 'H':
			switch(num){
				case '0': pin = PH_0; break; case '1': pin = PH_1; break; case '2': pin = PH_2; break; case '3': pin = PH_3; break;
				case '4': pin = PH_4; break; case '5': pin = PH_5; break; case '6': pin = PH_6; break; case '7': pin = PH_7; break;
			}
			break;
		case 'I':
			switch(num){
				case '0': pin = PI_0; break; case '1': pin = PI_1; break; case '2': pin = PI_2; break; case '3': pin = PI_3; break;
				case '4': pin = PI_4; break; case '5': pin = PI_5; break; case '6': pin = PI_6; break; case '7': pin = PI_7; break;
			}
			break;
		case 'J':
			switch(num){
				case '0': pin = PJ_0; break; case '1': pin = PJ_1; break; case '2': pin = PJ_2; break; case '3': pin = PJ_3; break;
				case '4': pin = PJ_4; break; case '5': pin = PJ_5; break; case '6': pin = PJ_6; break;
			}
			break;
		case 'K':
			switch(num){
				case '0': pin = PK_0; break; case '1': pin = PK_1; break; case '2': pin = PK_2; break; case '3': pin = PK_3; break;
				case '4': pin = PK_4; break; case '5': pin = PK_5; break; case '6': pin = PK_6; break;
			}
			break;
	}
	if(pin == NC){
		AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSG]: Invalid Pin Name");
		return;
	}
	// Remove debug info massage
	ConfigDebugInfo = 0;
	// Initial input control pin
	gpio_init(&gpio_test, pin);
	gpio_dir(&gpio_test, PIN_INPUT);     // Direction: Input
	gpio_mode(&gpio_test, PullUp);       // Pull-High
	val = gpio_read(&gpio_test);
	// Recover debug info massage
	ConfigDebugInfo = tConfigDebugInfo;
	AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSG] %c%c = %d", port, num, val);
}

void fATSC(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_DBG_MSG(AT_FLAG_OTA, AT_DBG_ALWAYS, "[ATSC]: _AT_SYSTEM_CLEAR_OTA_SIGNATURE_");
	sys_clear_ota_signature();
}

#if CONFIG_NFC_NORMAL
nfctag_t at_nfctag;
void fATSN(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};

	AT_DBG_MSG(AT_FLAG_NFC, AT_DBG_ALWAYS, "[ATSN]: _AT_SYSTEM_NFC_TEST_");
	if (!arg) {
		AT_DBG_MSG(AT_FLAG_NFC, AT_DBG_ALWAYS, "[ATSN] Usage: ATSN=COMMAND[PARAMETERS]");
		return;
	} else {
		argc = sys_parse_param(arg, argv);
		if(argc < 2){
			AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSN] Usage: ATSN=COMMAND[PARAMETERS]");
			return;
		}
	}

	switch(argv[1][0]) {
		case 'i':
			// init
			nfc_init(&at_nfctag, NULL);
			break;

		case 'f':
			// free
			nfc_free(&at_nfctag);
			break;

		default:
			AT_DBG_MSG(AT_FLAG_GPIO, AT_DBG_ALWAYS, "[ATSN] Usage: ATSN=COMMAND[PARAMETERS]");
			break;
	}
}
#endif

void fATSR(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_DBG_MSG(AT_FLAG_OTA, AT_DBG_ALWAYS, "[ATSR]: _AT_SYSTEM_RECOVER_OTA_SIGNATURE_");
	sys_recover_ota_signature();
}

void fATSs(void *arg)
{
	int argc = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_PRINTK("[ATS@]: _AT_SYSTEM_DBG_SETTING_");
	if(!arg){
		AT_PRINTK("[ATS@] Usage: ATS@=[LEVLE,FLAG]");
	}else{
		argc = sys_parse_param(arg, argv);
		if(argc == 3){
			char *ptmp;
			gDbgLevel = atoi(argv[1]);
			gDbgFlag = strtoul(argv[2], &ptmp, 16);
		}
	}
	AT_PRINTK("[ATS@] level = %d, flag = 0x%08X", gDbgLevel, gDbgFlag);
}

void fATSc(void *arg)
{
	int argc = 0, config = 0;
	char *argv[MAX_ARGC] = {0};
	
	AT_PRINTK("[ATS!]: _AT_SYSTEM_CONFIG_SETTING_");
	if(!arg){
		AT_PRINTK("[ATS!] Usage: ATS!=[CONFIG(0,1,2),FLAG]");
	}else{
		argc = sys_parse_param(arg, argv);
		if(argc == 3){
			char *ptmp;
			config = atoi(argv[1]);
			gDbgFlag = strtoul(argv[2], &ptmp, 16);
			if(config == 0)
				ConfigDebugErr = strtoul(argv[2], &ptmp, 16);
			if(config == 1)
				ConfigDebugInfo = strtoul(argv[2], &ptmp, 16);
			if(config == 2)
				ConfigDebugWarn = strtoul(argv[2], &ptmp, 16);
		}
	}
	AT_PRINTK("[ATS!] ConfigDebugErr  = 0x%08X", ConfigDebugErr);
	AT_PRINTK("[ATS!] ConfigDebugInfo = 0x%08X", ConfigDebugInfo);
	AT_PRINTK("[ATS!] ConfigDebugWarn = 0x%08X", ConfigDebugWarn);
}

void fATSx(void *arg)
{
	AT_PRINTK("[ATS?]: _AT_SYSTEM_HELP_");
}

log_item_t at_sys_items[] = {
	{"ATSD", fATSD,},	// Dump register
	{"ATSE", fATSE,},	// Edit register
	{"ATSA", fATSA,},	// MP ADC test
	{"ATSG", fATSG,},	// MP GPIO test
	{"ATSC", fATSC,},	// Clear OTA signature
	{"ATSR", fATSR,},	// Recover OTA signature
#if CONFIG_NFC_NORMAL
	{"ATSN", fATSN,},	// NFC test
#endif
	{"ATS@", fATSs,},	// Debug message setting
	{"ATS!", fATSc,},	// Debug config setting
	{"ATS?", fATSx,}	// Help
};

void at_sys_init(void)
{
	log_service_add_table(at_sys_items, sizeof(at_sys_items)/sizeof(at_sys_items[0]));
}

log_module_init(at_sys_init);
