/* mbed Microcontroller Library
 *******************************************************************************
 * Copyright (c) 2015, Realtek Semiconductor Corp.
 * All rights reserved.
 *
 * This module is a confidential and proprietary property of RealTek and
 * possession or use of this module requires written permission of RealTek.
 *******************************************************************************
 */

#ifndef MBED_EXT_I2S_API_EXT_H
#define MBED_EXT_I2S_API_EXT_H

#include "device.h"
#include "rtl8195a.h"
#include "hal_i2s.h"

#ifdef __cplusplus
extern "C" {
#endif

struct i2s_s {
	SAL_I2S_HND			I2sHdl;		// must first element
	RTK_I2S_OP			I2sOp;
	HAL_I2S_INIT_DAT	InitDat;
	SAL_I2S_USER_CB     I2SUserCallBack;
	char 	sampling_rate;
	char 	channel_num;
	char	word_length;
};

enum {
    SR_8KHZ = I2S_SR_8KHZ, 
    SR_16KHZ = I2S_SR_16KHZ,
    SR_24KHZ = I2S_SR_24KHZ,
	SR_32KHZ = I2S_SR_32KHZ,
    SR_48KHZ = I2S_SR_48KHZ,
	SR_96KHZ = I2S_SR_96KHZ,
	SR_7p35KHZ = I2S_SR_7p35KHZ,
	SR_11p02KHZ = I2S_SR_11p02KHZ,
	SR_22p05KHZ = I2S_SR_22p05KHZ,
	SR_29p4KHZ = I2S_SR_29p4KHZ,
	SR_44p1KHZ = I2S_SR_44p1KHZ,
	SR_88p2KHZ = I2S_SR_88p2KHZ
};

enum {
	CH_STEREO = I2S_CH_STEREO,
	CH_MONO = I2S_CH_MONO
};

enum {
	WL_16b = 0,
	WL_24b = 1
};

typedef struct i2s_s i2s_t;
	
void i2s_init(i2s_t *obj, int dev_idx, void (*tx_complete)(void*), void (*rx_complete)(void*));
void i2s_set_page(i2s_t *obj, int page_num, int page_size);
void i2s_set_direction(i2s_t *obj, int trx_type);
void i2s_set_param(i2s_t *obj, int channel_num, int rate, int word_len);
void i2s_deinit(i2s_t *obj);
void i2s_set_tx_buf(i2s_t *obj, void* tx_buf);
void i2s_set_rx_buf(i2s_t *obj, void* rx_buf);
void i2s_send(i2s_t *obj);
void i2s_recv(i2s_t *obj);

#ifdef __cplusplus
}
#endif


#endif