/* mbed Microcontroller Library
 *******************************************************************************
 * Copyright (c) 2015, Realtek Semiconductor Corp.
 * All rights reserved.
 *
 * This module is a confidential and proprietary property of RealTek and
 * possession or use of this module requires written permission of RealTek.
 *******************************************************************************
 */
#include "i2s_api.h"

void i2s_init(i2s_t *obj, int dev_idx, void (*tx_complete)(void*), void (*rx_complete)(void*))
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	SalI2SOpInit(pI2sOp);
	
	pI2sHdl->DevNum =  dev_idx;
	RtkI2SLoadDefault(pI2sHdl);
	
	obj->channel_num = pI2sHdl->I2SChNum;
	obj->sampling_rate = pI2sHdl->I2SRate;
	obj->word_length = pI2sHdl->I2SWordLen;
	
	pI2sHdl->pUserCB        = &obj->I2SUserCallBack;
	pI2sHdl->pUserCB->TXCCB = tx_complete;
	pI2sHdl->pUserCB->RXCCB = rx_complete;

	pI2sOp->Init(pI2sHdl);
}

void i2s_set_page(i2s_t *obj, int page_num, int page_size)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->DeInit(pI2sHdl);	

	pI2sHdl->I2SPageNum = page_num;
	pI2sHdl->I2SPageSize = page_size;
	
	pI2sOp->Init(pI2sHdl);
}

void i2s_set_direction(i2s_t *obj, int trx_type)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->DeInit(pI2sHdl);	
	pI2sHdl->I2STRxAct = trx_type;
	pI2sOp->Init(pI2sHdl);
}

void i2s_set_param(i2s_t *obj, int channel_num, int rate, int word_len)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->DeInit(pI2sHdl);
	
	pI2sHdl->I2SChNum = obj->channel_num = channel_num;
	pI2sHdl->I2SRate = obj->sampling_rate = rate;
	pI2sHdl->I2SWordLen = obj->word_length = word_len;
	
	pI2sOp->Init(pI2sHdl);
}

void i2s_deinit(i2s_t *obj)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->DeInit(pI2sHdl);
}

void i2s_set_tx_buf(i2s_t *obj, void* tx_buf)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;

	pI2sHdl->pTXBuf = tx_buf;
}

void i2s_set_rx_buf(i2s_t *obj, void* rx_buf)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;

	pI2sHdl->pRXBuf = rx_buf;
}

void i2s_send(i2s_t *obj)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->Send(pI2sHdl);
}

void i2s_recv(i2s_t *obj)
{
	PSAL_I2S_HND    pI2sHdl = &obj->I2sHdl;
	RTK_I2S_OP		*pI2sOp = &obj->I2sOp;
	
	pI2sOp->Receive(pI2sHdl);
}
