/* mbed Microcontroller Library
 *******************************************************************************
 * Copyright (c) 2014, Realtek
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************
 */
#include "cmsis.h"
#include "sys_api.h"
#include "flash_api.h"

#define OTA_Signature			"81958711"
#define OTA_Clear				"00000000"
#define OTA_Signature_len		8
#define OTA_Signature_offset	8
#define OTA_valid_offset		0x100000
#define printf					DiagPrintf

extern VOID HalJtagPinOff(VOID);

extern void HalInitLogUart(void);
extern void HalDeinitLogUart(void);

/**
 *  @brief   Turn off the JTAG function
 *
 *  @return  None
 *         
 */
void sys_jtag_off(void)
{
	HalJtagPinOff();
}

void sys_clear_ota_signature(void)
{
	flash_t		flash;
	u32			ota_offset;
	u8			signature[OTA_Signature_len+1];

	flash_stream_read(&flash, FLASH_SYSTEM_DATA_ADDR, 4, (u8*)&ota_offset);
	printf("\n\rOTA offset = 0x%08X", ota_offset);
	if(ota_offset < OTA_valid_offset){
		flash_stream_read(&flash, ota_offset+OTA_Signature_offset, OTA_Signature_len, signature);
		signature[OTA_Signature_len] = '\0';
		printf("\n\rSignature = %s", signature);
		if(!memcmp((char const*)signature, OTA_Signature, OTA_Signature_len)){
			memcpy((char*)signature, OTA_Clear, OTA_Signature_len);
			flash_stream_write(&flash, ota_offset+OTA_Signature_offset, OTA_Signature_len, signature);
			flash_stream_read(&flash, ota_offset+OTA_Signature_offset, OTA_Signature_len, signature);
			signature[OTA_Signature_len] = '\0';
			printf("\n\rSignature = %s", signature);
			printf("\n\rClear OTA signature success.");
		}
	}
}

void sys_recover_ota_signature(void)
{
	flash_t		flash;
	u32			ota_offset;
	u8			signature[OTA_Signature_len+1];
	u8*			pbuf;
	
	flash_stream_read(&flash, FLASH_SYSTEM_DATA_ADDR, 4, (u8*)&ota_offset);
	printf("\n\rOTA offset = 0x%08X", ota_offset);
	if(ota_offset < OTA_valid_offset){
		flash_stream_read(&flash, ota_offset+OTA_Signature_offset, OTA_Signature_len, signature);
		signature[OTA_Signature_len] = '\0';
		printf("\n\rSignature = %s", signature);
		if(!memcmp((char const*)signature, OTA_Clear, OTA_Signature_len)){
			// backup
			pbuf = RtlMalloc(FLASH_SECTOR_SIZE);
			if(!pbuf) return;
			flash_stream_read(&flash, ota_offset, FLASH_SECTOR_SIZE, pbuf);
			memcpy((char*)pbuf+OTA_Signature_offset, OTA_Signature, OTA_Signature_len);
			flash_erase_sector(&flash, FLASH_RESERVED_DATA_BASE);
			flash_stream_write(&flash, FLASH_RESERVED_DATA_BASE, FLASH_SECTOR_SIZE, pbuf);
			// Write
			flash_stream_read(&flash, FLASH_RESERVED_DATA_BASE, FLASH_SECTOR_SIZE, pbuf);
			flash_erase_sector(&flash, ota_offset);
			flash_stream_write(&flash, ota_offset, FLASH_SECTOR_SIZE, pbuf);
			flash_stream_read(&flash, ota_offset+OTA_Signature_offset, OTA_Signature_len, signature);
			signature[OTA_Signature_len] = '\0';
			printf("\n\rSignature = %s", signature);
			RtlMfree(pbuf, FLASH_SECTOR_SIZE);
			printf("\n\rRecover OTA signature success.");
		}
	}
}

void sys_log_uart_on(void)
{
    HalInitLogUart();
}

void sys_log_uart_off(void)
{
    HalDeinitLogUart();
}
