/*
 *  Routines to access hardware
 *
 *  Copyright (c) 2013 Realtek Semiconductor Corp.
 *
 *  This module is a confidential and proprietary property of RealTek and
 *  possession or use of this module requires written permission of RealTek.
 */

#include "device.h"
#include "analogin_api.h"



#define MBED_ADC_EXAMPLE_PIN_1    AD_1	// no pin out
#define MBED_ADC_EXAMPLE_PIN_2    AD_2	// HDK, A1
#define MBED_ADC_EXAMPLE_PIN_3    AD_3	// HDK, A2



#if defined (__ICCARM__)
analogin_t   adc1;
analogin_t   adc2;
analogin_t   adc3;
#else
volatile analogin_t   adc1;
volatile analogin_t   adc2;
volatile analogin_t   adc3;
#endif


void adc_delay(void)
{
    int i;
	for(i=0;i<1600000;i++)
		asm(" nop");
}

uint16_t adcdat1    = 0;
uint16_t adcdat2    = 0;
uint16_t adcdat3    = 0;

int32_t v_mv1;
int32_t v_mv2;
int32_t v_mv3;

/*
 * OFFSET:   value of measuring at 0.000v, value(0.000v)
 * GAIN_DIV: value(1.000v)-value(0.000v) or value(2.000v)-value(1.000v) or value(3.000v)-value(2.000v)
 *
 * MSB 12bit of value is valid, need to truncate LSB 4bit (0xABCD -> 0xABC). OFFSET and GAIN_DIV are truncated values.
 */
#define OFFSET 		0x298							
#define GAIN_DIV	0x34C							
#define AD2MV(ad) (((ad/16)-OFFSET)*1000/GAIN_DIV)	

VOID 
main (
    VOID
)
{
    analogin_init(&adc1, MBED_ADC_EXAMPLE_PIN_1);	// no pinout on HDK board
    analogin_init(&adc2, MBED_ADC_EXAMPLE_PIN_2);
    analogin_init(&adc3, MBED_ADC_EXAMPLE_PIN_3);
    
    for (;;){    
        adcdat1 = analogin_read_u16(&adc1);
        adcdat2 = analogin_read_u16(&adc2);
        adcdat3 = analogin_read_u16(&adc3);

	v_mv1 = AD2MV(adcdat1);
	v_mv2 = AD2MV(adcdat2);
	v_mv3 = AD2MV(adcdat3);

	printf("AD1:%x = %d mv, AD2:%x = %d mv, AD3:%x = %d mv\n", adcdat1, v_mv1, adcdat2, v_mv2, adcdat3, v_mv3); 
	adc_delay();

    }
}
